#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <systemctrl.h>
#include "fileman/filebrowser.h"
#include "system/usb.h"
#include "audio/mp3playerME.h"
#include <psppower.h>
#include <pspgu.h>
#include "gfx/graphics.h"
#include <psprtc.h>

#define STATE_MP3_NONE  0
#define STATE_MP3_PLAY  1
#define STATE_MP3_PAUSE 2
#define STATE_MP3_STOP  3

#define STATE_USB_ON    1
#define STATE_USB_OFF   0

PSP_MODULE_INFO("Mp3 ME Player",0, 1, 1); 
PSP_MAIN_THREAD_ATTR(PSP_THREAD_ATTR_USER);

int mp3_state = STATE_MP3_NONE;
int usb_state = STATE_USB_OFF;
struct fileInfo mp3Info;
fileIcon *self =NULL;
char out_str[480];
Image *cover = NULL;
int file_index_current_mp3 = 0;

// Prototypes from support.prx kernel module 0x1006

int imposeGetVolume();
int imposeGetBrightness();
int imposeSetVolume(int value); // 0-30
void initController();
void readButtons(SceCtrlData *pad_data, int count);

int exit_callback(int arg1, int arg2, void *common) {
	sceKernelExitGame();
	return 0;
}

int CallbackThread(SceSize args, void *argp) {

int cbid;
	
	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);
	sceKernelSleepThreadCB();
	return 0;
}

int SetupCallbacks(void) {

int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
		if(thid >= 0) {
			sceKernelStartThread(thid, 0, 0);
		}
	return thid;
}

int display_mp3_info(struct fileInfo *m) {
	if(cover)
		blitImageToScreen(0, 0, cover->imageWidth, cover->imageHeight, cover, 305, 23);  
   
	sprintf(out_str, "Title : %s", m->title);
	printTextScreen(DISPLAY_X, 210, out_str, BLUE);			 
 
	sprintf(out_str, "Album : %s", m->album);
	printTextScreen(DISPLAY_X, 220, out_str, BLUE);			 

	sprintf(out_str, "Year  : %s", m->year);
	printTextScreen(DISPLAY_X, 230, out_str, BLUE);			 

	sprintf(out_str, "Artist: %s", m->artist);
	printTextScreen(DISPLAY_X, 240, out_str, BLUE);			 
 
	sprintf(out_str, "Size : %s", commas((unsigned int)self->sizeB/1024));
	printTextScreen(360, 210, out_str, BLUE);			 
  
	sprintf(out_str, "kbit : %d", m->kbit);
	printTextScreen(360, 220, out_str, BLUE);			 

	sprintf(out_str, "Hz   : %d", m->hz);
	printTextScreen(360, 230, out_str, BLUE);			  

	sprintf(out_str, "Layer: %s", m->layer); 
	printTextScreen(360, 240, out_str, BLUE);	 

	return 0;
}

int start_mp3(fileIcon *file) {      
	if(file == NULL || file->fileType != FILE_TYPE_MP3)
		return -1;
		 
	switch(mp3_state) {
		case STATE_MP3_PLAY:
		case STATE_MP3_PAUSE:
						MP3ME_Stop();
		case STATE_MP3_STOP:
		case STATE_MP3_NONE:
						if(MP3ME_Load(file->filePath) == OPENING_OK) {
							mp3Info = MP3ME_GetInfo();
							file_index_current_mp3 = current;
							self = file;
							sprintf(out_str, "ms0:/MUSIC/covers/%s.jpg", mp3Info.album);
								if(cover)
									freeImage(cover);
 							cover = loadJpegImage(out_str);							
							
							if(MP3ME_Play() == 0)
								mp3_state = STATE_MP3_PLAY;
						}
						break;
											
	 }						  

	return 0;
} 

int main() {   
	SceCtrlData pad, oldpad;
	int quit = 0;
	pspTime local_time;

	SetupCallbacks();
	pspDebugScreenInit();	
    scePowerSetClockFrequency(80, 80, 40);	
	
	SceUID modid = pspSdkLoadStartModule("support.prx", PSP_MEMORY_PARTITION_KERNEL); 
		if(modid < 0)
			sceKernelExitGame();

	initController();
	imposeSetVolume(21);
	initGraphics();
	MP3ME_Init(1);
	USBInit();	 
	
	scan_mp3_files();
	
	while(!quit) {
		clearScreen(BLACK);

		if(mp3_state == STATE_MP3_PLAY) {
			if(MP3ME_EndOfStream() == 1) {
				MP3ME_Stop();
				mp3_state = STATE_MP3_STOP;
			}
	    }
	   	
	drawLineScreen(0, 15, 480, 15, YELLOW);
	drawLineScreen(0, 200, 480, 200, DARKGRAY);
	drawLineScreen(0, 255, 480, 255, YELLOW);	

	sprintf(out_str,"Bat:%d%%  CPU:%dmhz  USB:%s  Vol:%d%%  LCDBr:%d",scePowerGetBatteryLifePercent(), 
	scePowerGetCpuClockFrequencyInt(), usb_state?"On":"Off", (int)((double)imposeGetVolume() / 30.0 * 100.0),
	imposeGetBrightness());
	 
    printTextScreen(DISPLAY_X, 260, out_str, ORANGE);			 

	switch(mp3_state) {
		case STATE_MP3_PLAY:
					sprintf(out_str, "File %d of %d -> Playing %d", current, num_files_cwd-1, file_index_current_mp3); 
					break;
		case STATE_MP3_PAUSE:				
					sprintf(out_str, "File %d of %d -> Paused %d", current, num_files_cwd-1, file_index_current_mp3); 
					break;
		case STATE_MP3_STOP:
					sprintf(out_str, "File %d of %d -> Finished %d", current, num_files_cwd-1, file_index_current_mp3); 
					break;
		case STATE_MP3_NONE:
					sprintf(out_str, "File %d of %d", current, num_files_cwd-1); 
					break;		
	}
	
	printTextScreen(DISPLAY_X, 5, out_str, RED);	
	 	
	display_mp3_files();  	 
	 
	if(sceRtcGetCurrentClockLocalTime(&local_time) == 0) {
		sprintf(out_str, "%02d/%02d/%02d %02d:%02d:%02d", local_time.month, local_time.day, 
				local_time.year - 2000, local_time.hour, local_time.minutes, local_time.seconds);
	} else {
		strcpy(out_str, "Time Unavailable");
	}

	printTextScreen(368-strlen(out_str)-DISPLAY_X, 5, out_str, RED);

	if(mp3_state > STATE_MP3_NONE)
		display_mp3_info(&mp3Info);		  
	
	oldpad = pad;
	readButtons(&pad, 1);

 	if(pad.Buttons != oldpad.Buttons) {	 
	
		if(pad.Buttons & PSP_CTRL_START)
			quit = 1;
			
	    if(pad.Buttons & PSP_CTRL_RIGHT)
			move_down(5);
			
		if(pad.Buttons & PSP_CTRL_LEFT)
			move_up(5);

		if (pad.Buttons & PSP_CTRL_DOWN)
			move_down(1);
			
		if(pad.Buttons & PSP_CTRL_UP)
			move_up(1);
			
		if(pad.Buttons & PSP_CTRL_TRIANGLE)
			scan_mp3_files();
			
		if(pad.Buttons & PSP_CTRL_CROSS)
			start_mp3(run_file());
		 
		if(pad.Buttons & PSP_CTRL_SELECT) {
			if(usb_state == STATE_USB_OFF) {
				USBActivate();
				usb_state = STATE_USB_ON;
			} else { 
				USBDeactivate();
				usb_state = STATE_USB_OFF;
			}
		}
						
		if(pad.Buttons & PSP_CTRL_SQUARE) {
			if(mp3_state == STATE_MP3_PLAY) {
				MP3ME_Pause();
				mp3_state = STATE_MP3_PAUSE;
			} else {
				if(mp3_state == STATE_MP3_PAUSE) {
					MP3ME_Play();
					mp3_state = STATE_MP3_PLAY;
				}
			}
		}

   }
   
	sceDisplayWaitVblankStart();		
	flipScreen();
	   	  						
	}
	
	USBEnd();
	disableGraphics();
	sceKernelExitGame();
  
	return 0;
}



